/*
 Copyright (c) 2014 Shinji Tsuneyuki
 This file is distributed under the terms of the GNU General Public License version 3.
 */

#ifndef __DTQESP_H_INCLUDED
#define __DTQESPT_H_INCLUDED

#include <QtCore/QString>
#include "position.h"
#include "dtatoms.h"
#include "table.h"

#include <vector>
using namespace std;

class DTLattice;

struct DTQESP
{
  struct Control {
    QString calculation;
    QString restart_mode;
    double  forc_conv_thr; // MD criterion
    int     nstep; // MD step

    static Table forc_conv_thr_def;
  } control;
  
  struct System {
    int    ibrav;
    bool   use_celldm;
    double celldm[7];
    double A, B, C;
    double cosAB,cosBC,cosCA;
    double ecutwfc; // cutoff
    int    nbnd;
    int    nspin;

    static Table ecutwfc_def;
  } system;

  struct Electrons {
    int     electron_maxstep; // SCF step
    double  conv_thr; // SCF criterion
    double  mixing_beta; // SCF parameter

    static Table electron_maxstep_def;
    static Table conv_thr_def;
    static Table mixing_beta_def;
  } electrons;

  struct AtomicSpecies {
    struct Element {
      QString name;
      double  mass;
      QString psfile;
      Element( void ){}
      Element( const QString& name, double mass, const QString& psfile ){
	this->name   = name;
	this->mass   = mass;
	this->psfile = psfile;
      }
    };
    static vector<Element> velement_def;
  } atomicspecies;

  struct AtomicPositions
  {
    int unit;
    vector<DTAtom> atom;
  } atomicpositions;

  struct CellParameters{
    Position La;
    Position Lb;
    Position Lc;
    double   L;
    Position Ea;
    Position Eb;
    Position Ec;
  } cellparameters;

  struct KPoints{
    int    nkmesh[3];
    int    skmesh[3];
    int    nkband;
  } kpoints;

  DTQESP( void ){
  }
  ~DTQESP( void ){
    clear();
  }

  void convertCell( void );
  void convertAtom( void );


  void clear( void );
  QString guess( const QString& fname );
  bool load( const QString& fname );
  bool save( const DTLattice& lattice, const QString& fname ) const;
};

#endif // __DTQESP
